"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = useCreateReducerWithEffect;
exports.SideEffect = exports.UpdateWithSideEffect = exports.NoUpdate = exports.Update = void 0;

var _react = require("react");

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance"); }

function _iterableToArrayLimit(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

var Update = function Update(newState) {
  return function (_ref) {
    var setState = _ref.setState;
    setState(newState);
    return newState;
  };
};

exports.Update = Update;

var NoUpdate = function NoUpdate() {
  return function (_ref2) {
    var state = _ref2.state;
    return state;
  };
};

exports.NoUpdate = NoUpdate;

var UpdateWithSideEffect = function UpdateWithSideEffect(newState, sideEffect) {
  return function (_ref3) {
    var state = _ref3.state,
        setState = _ref3.setState,
        reducer = _ref3.reducer;
    var updatedState = Update(newState)({
      setState: setState
    });
    sideEffect(newState, function (action) {
      var reducerReturn = reducer(newState, action);
      reducerReturn({
        state: updatedState,
        setState: setState,
        reducer: reducer
      });
    });
  };
};

exports.UpdateWithSideEffect = UpdateWithSideEffect;

var SideEffect = function SideEffect(sideEffect) {
  return function (_ref4) {
    var state = _ref4.state,
        reducer = _ref4.reducer,
        setState = _ref4.setState;
    return sideEffect(function (action) {
      var reducerReturn = reducer(state, action);
      reducerReturn({
        state: state,
        setState: setState,
        reducer: reducer
      });
    });
  };
};

exports.SideEffect = SideEffect;

function useCreateReducerWithEffect(reducer, initialState) {
  var _useState = (0, _react.useState)(initialState),
      _useState2 = _slicedToArray(_useState, 2),
      state = _useState2[0],
      setState = _useState2[1];

  var dispatch = (0, _react.useMemo)(function () {
    return function (action) {
      var reducerReturn = reducer(state, action);

      if (typeof reducerReturn === "function") {
        reducerReturn({
          state: state,
          setState: setState,
          reducer: reducer
        });
      } else {
        setState(reducerReturn);
      }
    };
  }, [state, reducer]);
  return {
    state: state,
    dispatch: dispatch
  };
}